#include "crossroads.h"

void alloc_reporter (Service *s, int first) {
    int shmid;
    struct shmid_ds shmbuf;

    if (flag_foreground) {
	/* Fake it! */
	msg ("Obtaining reporting memory for service %s "
	     "(in memory segment, NOT in shared memory "
	     "since we're in debug mode)",
	     s->name);
	servicereport = xmalloc (sizeof(Servicereport));
	memset (servicereport, 0, sizeof(Servicereport));
	return;
    }
    
    /* Get an ID to work on, depending on the create mode.
       Get a corresponding semaphore too.
     */
    msg ("Obtaining reporting memory for service %s (key %d)",
	 s->name, s->shmkey);

    if (first) {
	/* First time 'round. We're allocating for the first time here. */
	if ( (shmid = shmget (s->shmkey, sizeof(Servicereport),
			      0644 | IPC_CREAT) ) >= 0)
	    msg ("Created new shm at id %d", shmid);
	else 
	    error ("Cannot create shared memory for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));

	/* Make sure that we're the owner of the segment. */
	if (shmctl (shmid, IPC_STAT, &shmbuf) == -1)
	    error ("Failed to get status of shared memory: %s",
		   strerror(errno));
	shmbuf.shm_perm.uid = getuid();
	shmbuf.shm_perm.gid = getgid();
	if (shmctl (shmid, IPC_SET, &shmbuf) == -1)
	    error ("Failed to set status of shared memory: %s",
		   strerror(errno));
	
	if ( (semid = semget (s->shmkey, 1, 0644 | IPC_CREAT)) >= 0 )
	    msg ("Created new sem at id %d", semid);
	else
	    error ("Cannot create semaphore for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));
    } else {
	/* Second time 'round. We're getting a handle on previously
	 * created shared memory and semaphores. */
	if ( (shmid = shmget (s->shmkey, sizeof(Servicereport),
			      0644) ) >= 0)
	    msg ("Got existing shm at id %d", shmid);
	else
	    error ("Cannot get shared memory for service %s, "
		   "key %d: %s. "
		   "Maybe crossroads isn't running, or the configuration "
		   "has changed.",
		   s->name, s->shmkey, strerror(errno));
	if ( (semid = semget (s->shmkey, 1, 0644)) >= 0 )
	    msg ("Got existing sem at id %d", semid);
	else
	    error ("Cannot get semaphore for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));
    }

    /* Attach to our memory segment. */
    if ( ((servicereport = shmat (shmid, 0, 0))) == (void *) -1 )
	error ("Cannot attach shared memory for service %s: %s",
	       s->name, strerror(errno));

    /* Zero it out if this is the first usage. */
    if (first)
	memset (servicereport, 0, sizeof(Servicereport));
}
