#include "crossroads.h"

static jmp_buf jmpbuf;
static int timeout;

static void alarmhandler (int sig) {
    alarm(0);
    msg ("Backend %s: connect timed out (%d sec)",
	 activeservice->backend[current_backend].name, CONNECT_TIMEOUT);
    longjmp (jmpbuf, 1);
}


int backend_connect (int is_waking) {
    int backend_sock;
    struct sockaddr_in servername;

    if ( (backend_sock = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	error ("Failed to create socket for backend communication: %s",
	       strerror(errno));

    if (init_sockaddr (&servername,
		       activeservice->backend[current_backend].server,
		       activeservice->backend[current_backend].port)) {
	/* The hostname is unusable.. */
	if (is_waking)
	    msg ("Unknown host %s",
		 activeservice->backend[current_backend].server);
	else
	    warning ("Unknown host %s",
		     activeservice->backend[current_backend].server);
	return (-1);
    }

    signal (SIGALRM, alarmhandler);
    timeout = 0;
    alarm (CONNECT_TIMEOUT);
    if (!setjmp (jmpbuf)) {
	/* First time around, try connecting */
	if (connect (backend_sock, (struct sockaddr *) &servername,
		     sizeof(servername)) < 0) {
	    /* This backend is unusable..
	     * Either connect() has failed (connection refused) or it got
	     * interrupted by the SIGALRM
	     */
	    alarm(0);
	    if (timeout) {
		if (is_waking)
		    msg ("Backend %s: not usable due to timeout",
			 activeservice->backend[current_backend].server);
		else
		    warning ("Backend %s: not usable due to timeout",
			     activeservice->backend[current_backend].server);
	    }
	    else
		msg ("Backend %s: cannot connect on port %d: %s",
		     activeservice->backend[current_backend].server,
		     activeservice->backend[current_backend].port,
		     strerror(errno));
	    close (backend_sock);
	    backend_sock = -2;
	}
	alarm(0);
    } else {
	/* Got here because of longjmp from the signal handler,
	 * this MUST be a timeout
	 */
	if (is_waking)
	    msg ("Backend %s: not usable due to timeout",
		 activeservice->backend[current_backend].server);
	else
	    warning ("Backend %s: not usable due to timeout",
			activeservice->backend[current_backend].server);
	backend_sock = -3;
    }

    signal (SIGALRM, SIG_DFL);
    return (backend_sock);
}
