%{
#include "crossroads.h"
#include "parser.h"

/* Required by flex */
static int yywrap () {
    return (1);
}

/* Lexer debugging related */
// #define LEXER_DEBUG
#ifdef LEXER_DEBUG
    static void lmsg (char const *x) {
        printf ("L: %s\n", x);
    }
    static void llmsg (char const *x, char const *y) {
        printf ("L: %s %s\n", x, y);
    }
#else
#   define lmsg(x)
#   define llmsg(x,y)
#endif
%}

%x stringstate commentstate

%%

#.*\n				{
				    lmsg ("comment");
				    yylineno++;
				}
\/\/.*\n			{
				    lmsg ("comment");
				    yylineno++;
				}
service				{
				    lmsg ("service");
				    return (SERVICE);
				}
bindto				{
				    lmsg ("bindto");
				    BEGIN (stringstate);
				    free (laststring);
				    laststring = 0;
				    return (BINDTO);
				}
port				{
				    lmsg ("port");
				    return (PORT);
				}
over				{
				    lmsg ("over");
				    return (OVER);
				}
shmkey				{
				    lmsg ("shmkey");
				    return (SHMKEY);
				}
backend				{
				    lmsg ("backend");
				    return (BACKEND);
				}
backlog				{
				    lmsg ("backlog");
				    return (BACKLOG);
				}
verbosity			{
				    lmsg ("verbosity");
				    return (VERBOSITY);
				}
sessiontimeout			{
				    lmsg ("sessiontimeout");
				    return (SESSIONTIMEOUT);
				}
maxclients			{
				    lmsg ("maxclients");
				    return (MAXCLIENTS);
				}
weight				{
				    lmsg ("weight");
				    return (WEIGHT);
				}
decay				{
				    lmsg ("decay");
				    return (DECAY);
				}
server				{
				    lmsg ("server");
				    BEGIN (stringstate);
				    free (laststring);
				    laststring = 0;
				    return (SERVER);
				}
dispatchmode			{
				    lmsg ("dispatchmode");
				    return (DISPATCHMODE);
				}
roundrobin			{
				    lmsg ("roundrobin");
				    return (ROUNDROBIN);
				}
random							{
				    lmsg ("random");
				    return (RANDOM);
				}
byduration							{
				    lmsg ("byduration");
				    return (BYDURATION);
				}
bysize							{
				    lmsg ("bysize");
				    return (BYSIZE);
				}
byorder				{
				    lmsg ("byorder");
				    return (BYORDER);
				}
revivinginterval		{
				    lmsg ("revivinginterval");
				    return (REVIVINGINTERVAL);
				}
dumptraffic			{
				    lmsg ("dumptraffic");
				    BEGIN (stringstate);
				    free (laststring);
				    laststring = 0;
				    return (DUMPTRAFFIC);
				}
onsuccess			{
				    lmsg ("onsuccess");
				    BEGIN (stringstate);
				    free (laststring);
				    laststring = 0;
				    return (ONSUCCESS);
				}
onfailure			{
				    lmsg ("onfailure");
				    BEGIN (stringstate);
				    free (laststring);
				    laststring = 0;
				    return (ONFAILURE);
				}
on|true|yes			{
				    lmsg ("on");
				    return (ON);
				}
off|false|no			{
				    lmsg ("off");
				    return (OFF);
				}
[a-zA-Z_][a-zA-Z0-9_]*		{
				    llmsg ("identifier", yytext);
				    return (IDENTIFIER);
				}
[0-9]+				{
				    llmsg ("number", yytext);
				    return (NUMBER);
				}
[ \t]+				{
				    lmsg ("space(s)");
				}
\n				{
				    lmsg ("newline");
				    yylineno++;
				}
.				{
				    llmsg ("lone char", yytext);
				    return (*yytext);
				}
\/\*				{
				    lmsg ("C-comment starts");
				    BEGIN(commentstate);
				}
<commentstate>\*\/		{
				    lmsg ("C-comment ends");
				    BEGIN(0);
				}
<commentstate>\n		{
				    yylineno++;
				}
<commentstate>.			;				

<stringstate>\"[^\"]*\"		{
				    llmsg ("string part", yytext);
				    laststring = xstrcat (laststring,
							  yytext);
				}
<stringstate>\'[^\']*\'		{
				    llmsg ("string part", yytext);
				    laststring = xstrcat (laststring,
							  yytext);
				}
<stringstate>;			{
				    BEGIN (0);
				    unput (';');
				    llmsg ("string", laststring);
				    return (STRING);
				}
<stringstate>" "		{
				    if (laststring) {
				        laststring = xstrcat (laststring,
							      yytext);
					llmsg ("string part", yytext);
				    }
				}
<stringstate>.			{
				    llmsg ("string part", yytext);
				    laststring = xstrcat (laststring,
							  yytext);
				}
<stringstate>\n			{
				    lmsg ("string part: ERROR - newline");
				    error ("Parse eror at line %d: "
					   "unterminated string, ';' "
					   "expected\n", yylineno + 1);
				}
