
#define EXTERN
#include "crossroads.h"

/* Explicit initializations. */

char *state_to_string_map [] = {	/* backend states as strings */
    "available",
    "UNAVAILABLE",
    "DOWN",
    "waking",
    "unknown",
    0,
};

int relevant_sigs[] = {			/* signals relevant to this app */
    SIGHUP,				/* either caught or ignored, */
    SIGINT,				/* depending on the stage */
    SIGPIPE,
    SIGTERM,
    0,
};

typedef struct {
    char *arg;				/* cmdline argument */
    int narg;				/* required remaining args */
    int needconf;			/* configuration file required? */
    void (*handler)(int ac, char **av);	/* action handler */
} Handler;

int main (int argc, char **argv) {
    int opt, i;    
    static Handler handler[] = {
	{ "sampleconf", 	0,	0,	sample_conf	},
	{ "services",		0,	1,	show_services	},
	{ "status",		0,	1,	show_status	},
	{ "stop",		0,	1,	stop_daemon	},
	{ "start", 		0,	1,	serve		},
	{ "restart",		0,	1,	restart 	},
	{ "tell", 		3,	1,	tell_service 	},
    };
    struct passwd *passwd;

    /* Remember original ac/av/ep */
    org_argc = argc;
    org_argv = argv;

    /* Parse options. */
    config_file = DEFAULT_CONF;
    while ( (opt = getopt (argc, argv, "?c:fhvi:u:")) > 0 )
	switch (opt) {
	case 'c':
	    config_file = optarg;
	    break;
	case 'f':
	    flag_foreground++;
	    break;
	case 'v':
	    flag_verbose++;
	    break;
	case 'i':
	    iflag_present++;
	    break;
	case 'u':
	    if (! (passwd = getpwnam (optarg)) )
		error ("Cannot fetch information for user '%s': %s",
		       optarg, strerror(errno));
	    uid = passwd->pw_uid;
	    break;
	case '?':
	case 'h':
	default:
	    usage ();
	}

    /* We need at last one argument: the action */
    if (optind >= argc)
	usage();

    /* Act on the argument. */
    for (i = 0; i < sizeof(handler) / sizeof(Handler); i++) {
	/* Match the handler action string with argv, and match the
	 * required remaining arguments. If they match..
	 * scan the configuration (if needed) and GO!
	 */
	if (!strcmp (argv[optind], handler[i].arg) &&
	    handler[i].narg == argc - optind - 1) {
	    if (handler[i].needconf) {
		msg ("Parsing configuration %s", config_file);
		if (! (yyin = fopen (config_file, "r")) )
		    error ("Cannot read configuration %s: %s",
			   config_file, strerror(errno));
		yyparse();
		fclose (yyin);
	    }
	    handler[i].handler (argc - optind, argv + optind + 1);
	    return (0);
	}
    }

    usage();
    return (1);
}
