#include "crossroads.h"

void runservice () {
    int listen_sock, pid;

    msg ("Service: %s (on port %d)",
	 activeservice->name, activeservice->port);
    
    /* Allocate the shared mem segment. */
    alloc_reporter(activeservice, 1);
    
    /* Now if required, go into the background. */
    if (!flag_foreground) {
	
	if ( (pid = fork()) < 0 ) {
	    /* Fork failed */
	    error ("Fork failure: %s", strerror(errno));
	} else if (pid) {
	    /* Parent branch */
	    msg ("Service %s detached as PID %d",
		 activeservice->name, pid);
	    return;
	} else {
	    /* Child branch */
	    if (uid) {
		if (seteuid(uid))
		    error ("Cannot assume effective uid %d: %s",
			   uid, strerror(errno));
	    }
		
	    set_program_title ("crossroads: service %s listening",
			       activeservice->name);
	    close (0);
	    close (1);
	    close (2);
	    daemonized++;
	    if ( (open ("/dev/null", O_RDONLY) < 0) ||
		 (open ("/dev/null", O_WRONLY) < 0) ||
		 (open ("/dev/null", O_WRONLY) < 0) )
		error ("Failed to reopen stdin/out/err on /dev/null");
	    if (setsid() < 0)
		error ("Failed to become seesion leader");
	    
	    /* Store our pid, so that 'crossroads stop' may kill us. */
	    lock_reporter();
	    servicereport->pid = getpid();
	    unlock_reporter();

	    /* Promote verbosity. */
	    flag_verbose = activeservice->verbosity;

	    /* We've forked for the first time */
	    program_stage = stage_waiting;
	}
    }

    /* In 'forever' mode, we create a server-side socket and ask tcpserve()
     * to service it. tcpserve() will return to us if there are no back
     * ends to service the request. In that case, we close the listening
     * socket (so that new clients get denied), take a nap, and redo.
     * Of course we only start listening if we have back ends at all...
     */
    while (1) {
	
	if (backend_available()) {
	    /* Create the socket, bind to port. */
	    listen_sock = make_socket (activeservice->port,
				       activeservice->bind);
	    msg ("Server-side network socket: %d", listen_sock);
	    
	    /* Start serving the port! */
	    tcpserve (listen_sock);
	    
	    /* tcpserve() returned -- must be because this service is out
	     * of back ends */
	    if (shutdown (listen_sock, SHUT_RD))
		error ("Failed to shut down server-side socket %d: %s",
		       listen_sock, strerror(errno));
	    if (close (listen_sock))
		error ("Failed to close server-side socket %d: %s",
		       listen_sock, strerror(errno));
	    
	    msg ("Taking a nap...");
	}
	
	sleep (SLEEP_TIME);
    }
}
