#include "crossroads.h"

void wakeup_handler () {
    int sock, i;
    
    /* Set the stage and any signals that we want. */
    program_stage = stage_retrying;
    for (i = 0; relevant_sigs[i]; i++)
	signal (relevant_sigs[i], interrupt);

    /* Forever, until a signal kills us, or Sol explodes, causing
     * massive electromagnetic pulses that fry my CPU.
     */
    while (1) {
	/* Wait for the alarm to go off. */
	sleep (activeservice->rev_interval);

	/* Now do our stuff. */
	for (current_backend = 0;
	     current_backend < activeservice->nbackend;
	     current_backend++)
	    if (servicereport->backendstate[current_backend].avail ==
		st_unavailable) {
		msg ("Wakeup service %s: trying to wake up backend %s",
		     activeservice->name,
		     activeservice->backend[current_backend].name);

		lock_reporter();
		servicereport->backendstate[current_backend].avail = st_waking;
		unlock_reporter();
		
		if ( (sock = backend_connect(1) ) >= 0) {
		    warning ("Backend %s of service %s has woken up",
			     activeservice->backend[current_backend].name,
			     activeservice->name);
		    lock_reporter();
		    servicereport->backendstate[current_backend].avail =
			st_available;
		    unlock_reporter();
		    close (sock);
		} else {
		    msg ("Backend %s of service %s is still unavailable",
			 activeservice->backend[current_backend].name,
			 activeservice->name);
		    lock_reporter();
		    servicereport->backendstate[current_backend].avail =
			st_unavailable;
		    unlock_reporter();
		}
	    }
    }
}
